VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CPalette"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

Private Type SectionHeader
    tag As String * 4
    sectionSize As Long
End Type

Private palData(255) As RGBQUAD

Public Function ReadPaletteData(lngFileNum As Long) As Long
    Dim numTagsLeft As Long
    Dim sectHead As SectionHeader
    Dim i As Long, temp As Long

    ' Initialize tag count
    numTagsLeft = -2
    ' Read all needed tags
    Do
        ' Decrease the tag count
        numTagsLeft = numTagsLeft - 1

        ' Read in the section header
        Get #lngFileNum, , sectHead

        ' Determine which type of section it was
        Select Case sectHead.tag
        Case "PPAL"
            ' Error check the section size
            If sectHead.sectionSize <> 1048 Then
                Debug.Print "CPalette: Warning! 'PPAL' section size is not 1048"
            End If
        Case "RIFF"
            Debug.Print "CPalette: Warning! 'RIFF' tag unhandled"
        Case "head"
            ' Error check the section size
            If sectHead.sectionSize <> 4 Then
                Debug.Print "CPalette: Warning! 'head' section size is not 4"
            End If
            ' Read numTagsLeft (4 bytes of data)
            Get #lngFileNum, , numTagsLeft
        Case "data"
            ' Error check the section size
            If sectHead.sectionSize <> 1024 Then
                Debug.Print "CPalette: Warning! 'data' section size is not 1024"
            End If
            ' Read the palette data
            Get #lngFileNum, , palData()
        Case "pspl"
            Debug.Print "CPalette: Warning! 'pspl' tag unhandled"
        Case "ptpl"
            Debug.Print "CPalette: Warning! 'ptpl' tag unhandled"
        Case Else
            Debug.Print "CPalette: Warning! Unrecognized section tag: " + sectHead.tag
            ReadPaletteData = -1    ' Error
            Exit Function
        End Select
    Loop While numTagsLeft <> 0
    
    ' Reverse the color components of the palette data
    ' (For some reason, the custom Outpost 2 format stores
    '  it backwards from the standard Windows format)
    For i = 0 To 255
        temp = palData(i).rgbBlue
        palData(i).rgbBlue = palData(i).rgbRed
        palData(i).rgbRed = temp
    Next i
            
    ' Return success
    ReadPaletteData = 0
End Function

Friend Sub GetPaletteData(paletteData() As RGBQUAD)
    Dim i As Long
    
    For i = 0 To 255
        paletteData(i) = palData(i)
    Next i
End Sub

Friend Sub SetPaletteData(index As Long, palEntry As RGBQUAD)
    If index > 0 And index <= 255 Then
        palData(index) = palEntry
    End If
End Sub
