VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "CPrtFile"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit


' Structures for the image port of the .prt file (second section)

Private Type ImageInfo
    ScanLineByteWidth As Long   ' Pitch
    DataPtr As Long
    Height As Long
    Width As Long
    Type As Integer
    PaletteNum As Integer
End Type
' The field "Type" has the following values/meanings
' 0 - menu graphic
' 1 - in game graphic
' 2
' 3
' 4 - Unit shadows - (1 bpp)
' 5 - Unit shadows - (1 bpp)
' ... more, many more


' Structures for the Animation section of the .prt file (third section)

Private Type Rect
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
End Type

' Animation header as seen in the file.
' This data is follow by the data for each frame
Private Type AnimationHeaderInfo
    unknown As Long             ' Set to 1 **TODO** Find out use
    SelectionBox As Rect
    PixelXDisplacement As Long
    PixelYDisplacement As Long
    unknown2 As Long            ' Set to 0 (usually) **TODO** Find out use
    NumAnimationFrames As Long
End Type

Private Type AnimationAppendixInfo  ' Name this better? **TODO**
    unknown1 As Long
    unknown2 As Long
    unknown3 As Long
    unknown4 As Long
End Type

Private Type FrameHeaderInfo
    numFrameComponents As Byte
    unknown As Byte
End Type


Private Type FrameOptionalInfo
    unknown1 As Byte
    unknown2 As Byte
    unknown3 As Byte
    unknown4 As Byte
End Type

Private Type FrameComponentInfo
    ImageInfoIndex As Integer   ' Index in the image() array for this component
    Unused As Byte              ' Set to 0xFF (Unused/Ignored by Outpost2)
    FrameComponentIndex As Byte ' ??? What's the point?
    PixelXOffset As Integer     ' Offset to left edge of image
    PixelYOffset As Integer     ' Offset to top edge of image
End Type


' Structures to hold the animation data in memory

Private Type AnimationInfo
    Header As AnimationHeaderInfo
    StartingFrameIndex As Long
    NumUnknown As Long
    StartingUnknownIndex As Long
End Type

Private Type FrameInfo
    Header As FrameHeaderInfo
    Optional As FrameOptionalInfo
    StartingFrameComponentIndex As Long
End Type



' PRT data
Private numLoadedPalettes As Long           ' Total number of palettes for all images
Private palette() As CPalette               ' Array of all palettes needed for the images
Private numLoadedImages As Long             ' Total number of stored images
Private images() As ImageInfo               ' Array of image info structs describing all images in this file
Private numLoadedAnimations As Long         ' Number of animation sequences
Private numLoadedFrames As Long             ' Total number of frames in all animations
Private numLoadedFrameComponents As Long    ' Total number of all components of all frames of all animations
Private numLoadedFrameOptional As Long      ' Additional info attached to select frames
Private animations() As AnimationInfo       ' Array describing all animation sequences
Private frames() As FrameInfo               ' Array describing all frames of animation
Private frameComponents() As FrameComponentInfo ' Array describing all components of each frame of animation

' Bitmap data
Private bmFileHeader As BITMAPFILEHEADER
Private bmInfo As BITMAPINFO
Private bitmapData() As Byte

Public Function OpenBitmapFile(strFileName As String) As Long
    Dim lngFileNum As Long

    ' Check if the file exists
    If Len(Dir(strFileName)) = 0 Then
        ' Error, file doesn't exist
        OpenBitmapFile = -1
        Exit Function
    End If
    
    ' Open the bitmap file
    lngFileNum = FreeFile
    Open strFileName For Binary As #lngFileNum
    
    ' Read in the bitmap data
    OpenBitmapFile = ReadBitmapData(lngFileNum)
    
    ' Close the bitmap file
    Close #lngFileNum
End Function

Public Function ReadBitmapData(lngFileNum As Long) As Long
    ' Read the bitmap header
    Get #lngFileNum, , bmFileHeader
    Get #lngFileNum, , bmInfo

    ' Validate file size
    If LOF(lngFileNum) <= 0 Then
        ReadBitmapData = -1
        Exit Function
    End If
    
    ' Allocate space for the bitmap pixel data
    ReDim bitmapData(LOF(lngFileNum) - Loc(lngFileNum)) As Byte
    ' Load the bitmap pixel data
    Get #lngFileNum, , bitmapData()

    ' Return success
    ReadBitmapData = 0
End Function

Public Function OpenPrtFile(strFileName As String) As Long
    Dim lngFileNum As Long
    
    ' Check if the file exists
    If Len(Dir(strFileName)) = 0 Then
        ' Error, file doesn't exist
        OpenPrtFile = -1
        Exit Function
    End If
    
    ' Open the file for reading
    lngFileNum = FreeFile
    Open strFileName For Binary As #lngFileNum
    
    ' Read in the PRT data (and assign return code)
    OpenPrtFile = ReadPrtData(lngFileNum)
    
    ' Close the file
    Close #lngFileNum
End Function

Public Function ReadPrtData(lngFileNum As Long) As Long
    Dim i As Long, j As Long, k As Long
    Dim strTag As String * 4

    ' Read in the palette section
    ' ***************************

    ' Read the palette section header tag
    Get #lngFileNum, , strTag
    If strTag <> "CPAL" Then
        ReadPrtData = -1    ' Return failure
        Exit Function
    End If

    ' Read the number of palettes
    Get #lngFileNum, , numLoadedPalettes
    
    ' Allocate the palette array
    ReDim palette(numLoadedPalettes - 1) As CPalette
    ' Read in all the palette info
    For i = 0 To numLoadedPalettes - 1
        ' Create a new palette object
        Set palette(i) = New CPalette
        ' Read in the palette data
        If palette(i).ReadPaletteData(lngFileNum) <> 0 Then
            ' Error reading in palette data. Abort.
            Debug.Print "CPrtFile: Error reading in palette #" + Format$(i)
            ReadPrtData = -1
            Exit Function
        End If
        
        ' **DEBUG** Palette remapping test code
        If i > 0 Then
            For j = 1 To 23
                'palette(i).SetPaletteData j, pal(i)
            Next j
        End If
    Next i
    
    
    ' Read in the images info
    ' ***********************
    
    ' Read in the number of image info structs
    Get #lngFileNum, , numLoadedImages
    
    ' Allocate array to hold image structs
    ReDim images(numLoadedImages - 1) As ImageInfo
    ' Read in the image info structs
    Get #lngFileNum, , images()
    
    ' Read in the Animation info
    ' ***********************
    
    ' Read in the structure counts
    Get #lngFileNum, , numLoadedAnimations
    Get #lngFileNum, , numLoadedFrames
    Get #lngFileNum, , numLoadedFrameComponents
    Get #lngFileNum, , numLoadedFrameOptional
    
    ' Allocate space for the data
    ReDim animations(numLoadedAnimations - 1)
    ReDim frames(numLoadedFrames - 1)
    ReDim frameComponents(numLoadedFrameComponents - 1)
    
    Dim lngCurrentFrame As Long
    Dim lngCurrentFrameComponent As Long
    
    ' Read in the Animation data
    For i = 0 To numLoadedAnimations - 1
        ' Read the animation header
        Get #lngFileNum, , animations(i).Header
        
        ' Set the starting frame index
        animations(i).StartingFrameIndex = lngCurrentFrame

        ' Read in all frame info
        For j = 0 To animations(i).Header.NumAnimationFrames - 1
            ' Read in all frame component info
            Get #lngFileNum, , frames(lngCurrentFrame).Header
            
            ' Check if first block of optional data needs to be read in
            If (frames(lngCurrentFrame).Header.numFrameComponents And &H80) = &H80 Then
                ' Read in the optional data
                Get #lngFileNum, , frames(lngCurrentFrame).Optional.unknown1
                Get #lngFileNum, , frames(lngCurrentFrame).Optional.unknown2
            End If
            ' Check if second block of optional data needs to be read in
            If (frames(lngCurrentFrame).Header.unknown And &H80) = &H80 Then
                ' Read in the optional data
                Get #lngFileNum, , frames(lngCurrentFrame).Optional.unknown3
                Get #lngFileNum, , frames(lngCurrentFrame).Optional.unknown4
            End If
            
            ' Set the starting frame component index
            frames(lngCurrentFrame).StartingFrameComponentIndex = lngCurrentFrameComponent
            
            For k = 0 To (frames(lngCurrentFrame).Header.numFrameComponents And &H7F) - 1
                ' Read the frame component
                Get #lngFileNum, , frameComponents(lngCurrentFrameComponent)
                lngCurrentFrameComponent = lngCurrentFrameComponent + 1
            Next k
            lngCurrentFrame = lngCurrentFrame + 1
        Next j
        
        ' Read number of (appedicies?)
        Get #lngFileNum, , animations(i).NumUnknown

        ' Read in all (appendix?) data
        For j = 0 To animations(i).NumUnknown - 1
            ' Read in the (appendix?) structure (and forget it! ;)  **TODO** Improve
            Dim temp As AnimationAppendixInfo
            Get #lngFileNum, , temp
        Next j
    Next i


    ' Check if the correct amount of data was loaded
    If numLoadedFrames <> lngCurrentFrame Then
        Debug.Print "Incorrect number of frames loaded"
    End If
    If numLoadedFrameComponents <> lngCurrentFrameComponent Then
        Debug.Print "Incorrect number of frame components loaded"
    End If

    ' Return success
    ReadPrtData = 0
End Function


' Image access functions
' **********************

Public Property Get numImages() As Long
    numImages = numLoadedImages
End Property

Public Property Get ImageWidth(imageNum As Long) As Long
    ImageWidth = images(imageNum).Width
End Property

Public Property Get ImageHeight(imageNum As Long) As Long
    ImageHeight = images(imageNum).Height
End Property

Public Property Get ImageType(imageNum As Long) As Long
    ImageType = images(imageNum).Type
End Property

Public Property Get ImagePalette(imageNum As Long) As Long
    ImagePalette = images(imageNum).PaletteNum
End Property

' Animation access functions
' **************************
' **TODO** Finish writing these/add more

Public Property Get NumAnimations() As Long
    NumAnimations = numLoadedAnimations
End Property

Public Property Get NumFrames(animationIndex As Long) As Long
    NumFrames = animations(animationIndex).Header.NumAnimationFrames
End Property

' Pasting functions
' *****************

Public Function Paste(destDC As Long, destX As Long, destY As Long, imageNum As Long) As Long
    ' Fill in temp fields
    bmInfo.bmiHeader.biWidth = images(imageNum).Width
    bmInfo.bmiHeader.biHeight = -images(imageNum).Height
    
    ' **TODO** Fix this cheap hack to adjust bit depth
    If images(imageNum).Type < 4 Then
        bmInfo.bmiHeader.biBitCount = 8
    ElseIf images(imageNum).Type > 5 Then
        bmInfo.bmiHeader.biBitCount = 8
    Else
        bmInfo.bmiHeader.biBitCount = 1
    End If

    ' Fill in the palette
    palette(images(imageNum).PaletteNum).GetPaletteData bmInfo.bmiColors()

    ' Paste the image
    Paste = SetDIBitsToDevice(destDC, destX, destY, images(imageNum).Width, images(imageNum).Height, 0, 0, 0, images(imageNum).Height, bitmapData(images(imageNum).DataPtr), bmInfo, DIB_RGB_COLORS)
End Function

